#include "beatfish.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <gdk/gdkx.h>
#include <gtk/gtk.h>

#include <Evas.h>
#include <Evas_Engine_Software_X11.h>

static void
evasgtk2_expose (GtkWidget * widget, GdkEventExpose * event, Evas * evas)
{
  GdkRectangle * a = &event->area;
  evas_damage_rectangle_add(evas, a->x, a->y, a->width, a->height);
}

static void
evasgtk2_configure (GtkWidget * widget, GdkEventConfigure * event, Evas * evas)
{
  evas_output_size_set(evas, event->width, event->height);
}

static void
evasgtk2_button_press (GtkWidget * widget, GdkEventButton * event, Evas * evas)
{
  evas_event_feed_mouse_move (evas, event->x, event->y);
  evas_event_feed_mouse_down (evas, event->button);
}

static void
evasgtk2_button_release (GtkWidget * widget, GdkEventButton * event,
			 Evas * evas)
{
  evas_event_feed_mouse_move (evas, event->x, event->y);
  evas_event_feed_mouse_up (evas, event->button);
}

static void
evasgtk2_motion (GtkWidget * widget, GdkEventMotion * event, Evas * evas)
{
  evas_event_feed_mouse_move (evas, event->x, event->y);
}

static void
evasgtk2_delete (GtkWidget * widget, Evas * evas)
{
  evas_free (evas);
}

static Evas *
evasgtk2_attach (GtkWidget * widget, double w, double h)
{
  Display * disp;
  Window win;
  Evas * evas;
  
  disp = gdk_x11_drawable_get_xdisplay (GDK_DRAWABLE(widget->window));
  win = gdk_x11_drawable_get_xid (GDK_DRAWABLE(widget->window));

  gdk_window_set_events (widget->window,
			 GDK_EXPOSURE_MASK | GDK_STRUCTURE_MASK |
			 GDK_POINTER_MOTION_MASK |
			 GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);

  evas = evas_new();
  evas_output_method_set(evas, evas_render_method_lookup("software_x11"));
  evas_output_size_set(evas, w, h);
  evas_output_viewport_set(evas, 0, 0, w, h);
  {
    Evas_Engine_Info_Software_X11 *einfo;

    einfo = (Evas_Engine_Info_Software_X11 *) evas_engine_info_get(evas);

    /* the following is specific to the engine */
    einfo->info.display = disp;
    einfo->info.visual = DefaultVisual(disp, DefaultScreen(disp));
    einfo->info.colormap = DefaultColormap(disp, DefaultScreen(disp));
    einfo->info.drawable = win;
    einfo->info.depth = DefaultDepth(disp, DefaultScreen(disp));
    einfo->info.rotation = 0;
    einfo->info.debug = 0;
    evas_engine_info_set(evas, (Evas_Engine_Info *) einfo);
  }

  g_signal_connect (G_OBJECT(widget), "expose_event",
		    G_CALLBACK(evasgtk2_expose), evas);
  g_signal_connect (G_OBJECT(widget), "configure_event",
		    G_CALLBACK(evasgtk2_configure), evas);
  g_signal_connect (G_OBJECT(widget), "button_press_event",
		    G_CALLBACK(evasgtk2_button_press), evas);
  g_signal_connect (G_OBJECT(widget), "button_release_event",
		    G_CALLBACK(evasgtk2_button_release), evas);
  g_signal_connect (G_OBJECT(widget), "motion_notify_event",
		    G_CALLBACK(evasgtk2_motion), evas);

  g_signal_connect (G_OBJECT(widget), "destroy_event",
		    G_CALLBACK(evasgtk2_delete), evas);

  return evas;
}

/* app from here down */

static gboolean going = TRUE;

gint
update_cursor_timeout (Display * disp)
{
  if (going) {
    update_cursor ();
    evas_render(evas);
    /*XFlush (disp);*/
  }

  return going;
}

gint
delete_canvas (GtkWidget * widget)
{
  gtk_widget_destroy (widget);
  going = FALSE;

  return FALSE;
}

int
main(int argc, char **argv)
{
  Display * disp;
  GtkWidget * window;

  GtkWidget * evasgtk2;

  gtk_init (&argc, &argv);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (window, gdk_screen_width() / 2,
			       gdk_screen_height() / 2);
  gtk_widget_show (window);

  disp = gdk_x11_drawable_get_xdisplay (GDK_DRAWABLE(window->window));

  evasgtk2 = gtk_drawing_area_new ();
  gtk_container_add (GTK_CONTAINER (window), evasgtk2);
  gtk_widget_show (evasgtk2);

  evas = evasgtk2_attach (evasgtk2, win_w, win_h);

  setup ();

  gtk_timeout_add (10, (GtkFunction)update_cursor_timeout, disp);

  /*gtk_timeout_add (1000, (GtkFunction)delete_canvas, evasgtk2);*/

  gtk_main ();

  return 0;
}
